@@ -33,7 +33,7 @@
 
 #include "omc_config.h"
 
-#ifdef WITH_UMFPACK
+#ifdef WITH_SUITESPARSE
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,6 +41,7 @@
 #include "simulation_data.h"
 #include "simulation/simulation_info_json.h"
 #include "util/omc_error.h"
+#include "util/parallel_helper.h"
 #include "omc_math.h"
 #include "util/varinfo.h"
 #include "model_help.h"
@@ -48,7 +49,6 @@
 #include "linearSystem.h"
 #include "linearSolverKlu.h"
 
-
 static void printMatrixCSC(int* Ap, int* Ai, double* Ax, int n);
 static void printMatrixCSR(int* Ap, int* Ai, double* Ax, int n);
 
@@ -123,15 +123,26 @@ static int getAnalyticalJacobian(DATA* data, threadData_t *threadData,
   LINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->linearSystemData[sysNumber]);
 
   const int index = systemData->jacobianIndex;
-  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[systemData->jacobianIndex]);
-  ANALYTIC_JACOBIAN* parentJacobian = systemData->parentJacobian;
+  ANALYTIC_JACOBIAN* jacobian = systemData->parDynamicData[omc_get_thread_num()].jacobian;
+  ANALYTIC_JACOBIAN* parentJacobian = systemData->parDynamicData[omc_get_thread_num()].parentJacobian;
 
   int nth = 0;
-  int nnz = jacobian->sparsePattern->numberOfNoneZeros;
+  int nnz = jacobian->sparsePattern->numberOfNonZeros;
 
-  for(i=0; i < jacobian->sizeRows; i++)
+  if (jacobian->constantEqns != NULL) {
+    jacobian->constantEqns(data, threadData, jacobian, parentJacobian);
+  }
+
+  for(i=0; i < jacobian->sparsePattern->maxColors; i++)
   {
-    jacobian->seedVars[i] = 1;
+    /* activate seed variable for the corresponding color */
+    for(ii=0; ii < jacobian->sizeCols; ii++)
+    {
+      if(jacobian->sparsePattern->colorCols[ii]-1 == i)
+      {
+        jacobian->seedVars[ii] = 1;
+      }
+    }
 
     ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, parentJacobian);
 
@@ -139,20 +150,18 @@ static int getAnalyticalJacobian(DATA* data, threadData_t *threadData,
     {
       if(jacobian->seedVars[j] == 1)
       {
-        ii = jacobian->sparsePattern->leadindex[j];
-        while(ii < jacobian->sparsePattern->leadindex[j+1])
+        nth = jacobian->sparsePattern->leadindex[j];
+        while(nth < jacobian->sparsePattern->leadindex[j+1])
         {
-          l  = jacobian->sparsePattern->index[ii];
-          systemData->setAElement(i, l, -jacobian->resultVars[l], nth, (void*) systemData, threadData);
+          l  = jacobian->sparsePattern->index[nth];
+          systemData->setAElement(j, l, -jacobian->resultVars[l], nth, (void*) systemData, threadData);
           nth++;
-          ii++;
         }
+        /* de-activate seed variable for the corresponding color */
+        jacobian->seedVars[j] = 0;
       }
     }
-    /* de-activate seed variable for the corresponding color */
-    jacobian->seedVars[i] = 0;
   }
-
   return 0;
 }
 
@@ -179,7 +188,7 @@ int solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 {
   void *dataAndThreadData[2] = {data, threadData};
   LINEAR_SYSTEM_DATA* systemData = &(data->simulationInfo->linearSystemData[sysNumber]);
-  DATA_KLU* solverData = (DATA_KLU*)systemData->solverData[0];
+  DATA_KLU* solverData = (DATA_KLU*)systemData->parDynamicData[omc_get_thread_num()].solverData[0];
   _omc_scalar residualNorm = 0;
 
   int i, j, status = 0, success = 0, n = systemData->size, eqSystemNumber = systemData->equationIndex, indexes[2] = {1,eqSystemNumber};
@@ -217,7 +226,8 @@ int solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 
     /* calculate vector b (rhs) */
     memcpy(solverData->work, aux_x, sizeof(double)*solverData->n_row);
-    residual_wrapper(solverData->work, systemData->b, dataAndThreadData, sysNumber);
+
+  residual_wrapper(solverData->work, systemData->parDynamicData[omc_get_thread_num()].b, dataAndThreadData, sysNumber);
   }
   tmpJacEvalTime = rt_ext_tp_tock(&(solverData->timeClock));
   systemData->jacobianTime += tmpJacEvalTime;
@@ -240,11 +250,13 @@ int solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
     messageClose(LOG_LS_V);
 
     for (i=0; i<solverData->n_row; i++)
-      infoStreamPrint(LOG_LS_V, 0, "b[%d] = %e", i, systemData->b[i]);
+    {
+      // ToDo Rework stream prints like this one to work in parallel regions
+      infoStreamPrint(LOG_LS_V, 0, "b[%d] = %e", i, systemData->parDynamicData[omc_get_thread_num()].b[i]);
+    }
   }
   rt_ext_tp_tick(&(solverData->timeClock));
 
-
   /* symbolic pre-ordering of A to reduce fill-in of L and U */
   if (0 == solverData->numberSolving)
   {
@@ -276,11 +288,11 @@ int solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 
   if (0 == solverData->common.status){
     if (1 == systemData->method){
-      if (klu_solve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->b, &solverData->common)){
+      if (klu_solve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->parDynamicData[omc_get_thread_num()].b, &solverData->common)){
         success = 1;
       }
     } else {
-      if (klu_tsolve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->b, &solverData->common)){
+      if (klu_tsolve(solverData->symbolic, solverData->numeric, solverData->n_col, 1, systemData->parDynamicData[omc_get_thread_num()].b, &solverData->common)){
         success = 1;
       }
     }
@@ -294,7 +306,7 @@ int solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
     if (1 == systemData->method){
       /* take the solution */
       for(i = 0; i < solverData->n_row; ++i)
-        aux_x[i] += systemData->b[i];
+        aux_x[i] += systemData->parDynamicData[omc_get_thread_num()].b[i];
 
       /* update inner equations */
       residual_wrapper(aux_x, solverData->work, dataAndThreadData, sysNumber);
@@ -308,7 +320,7 @@ int solveKlu(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
       }
     } else {
       /* the solution is automatically in x */
-      memcpy(aux_x, systemData->b, sizeof(double)*systemData->size);
+      memcpy(aux_x, systemData->parDynamicData[omc_get_thread_num()].b, sizeof(double)*systemData->size);
     }
 
     if (ACTIVE_STREAM(LOG_LS_V))
