@@ -354,11 +354,17 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
   NONLINEAR_SYSTEM_DATA *nonlinsys = data->simulationInfo->nonlinearSystemData;
   struct dataSolver *solverData;
   struct dataMixedSolver *mixedSolverData;
+  modelica_boolean someSmallDensity = 0;  /* pretty dumping of flag info */
+  modelica_boolean someBigSize = 0;       /* analogous to someSmallDensity */
 
   infoStreamPrint(LOG_NLS, 1, "initialize non-linear system solvers");
   infoStreamPrint(LOG_NLS, 0, "%ld non-linear systems", data->modelData->nNonLinearSystems);
+
+  /* set the default nls linear solver depending on the default nls method */
   if (data->simulationInfo->nlsLinearSolver == NLS_LS_DEFAULT) {
 #if !defined(OMC_MINIMAL_RUNTIME)
+    /* kinsol works best with KLU,
+       they are both sparse so it makes sense to use them together */
     if (data->simulationInfo->nlsMethod == NLS_KINSOL) {
       data->simulationInfo->nlsLinearSolver = NLS_LS_KLU;
     } else {
@@ -389,19 +395,6 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
       }
     }
 
-#if !defined(OMC_MINIMAL_RUNTIME)
-    if (nonlinsys[i].isPatternAvailable)
-    {
-      nnz = nonlinsys[i].sparsePattern->numberOfNoneZeros;
-
-      if(nnz/(double)(size*size)<=nonlinearSparseSolverMaxDensity && size >= nonlinearSparseSolverMinSize)
-      {
-        data->simulationInfo->nlsMethod = NLS_KINSOL;
-        infoStreamPrint(LOG_STDOUT, 0, "Using sparse solver kinsol for nonlinear system %d,\nbecause density of %.2f remains under threshold of %.2f and size of %d exceeds threshold of %d.\nThe maximum density and the minimal system size for using sparse solvers can be specified\nusing the runtime flags '<-nlsMaxDensity=value>' and '<-nlsMinSize=value>'.", i, nnz/(double)(size*size), nonlinearSparseSolverMaxDensity, size, nonlinearSparseSolverMinSize);
-      }
-    }
-#endif
-
     /* allocate system data */
     nonlinsys[i].nlsx = (double*) malloc(size*sizeof(double));
     nonlinsys[i].nlsxExtrapolation = (double*) malloc(size*sizeof(double));
@@ -434,7 +427,49 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
     }
 #endif
 
-    switch(data->simulationInfo->nlsMethod)
+    /* check if the system is sparse enough to use kinsol
+       it is considered sparse if
+         * the density (nnz/size^2) is less than a threshold or
+         * the size is bigger than a threshold */
+    nonlinsys[i].nlsMethod = data->simulationInfo->nlsMethod;
+    nonlinsys[i].nlsLinearSolver = data->simulationInfo->nlsLinearSolver;
+#if !defined(OMC_MINIMAL_RUNTIME)
+    if (nonlinsys[i].isPatternAvailable && data->simulationInfo->nlsMethod != NLS_KINSOL)
+    {
+      nnz = nonlinsys[i].sparsePattern->numberOfNonZeros;
+
+      if (nnz/(double)(size*size) < nonlinearSparseSolverMaxDensity) {
+        nonlinsys[i].nlsMethod = NLS_KINSOL;
+        nonlinsys[i].nlsLinearSolver = NLS_LS_KLU;
+        someSmallDensity = 1;
+        if (size > nonlinearSparseSolverMinSize) {
+          someBigSize = 1;
+          infoStreamPrint(LOG_STDOUT, 0,
+                          "Using sparse solver kinsol for nonlinear system %d (%ld),\n"
+                          "because density of %.2f remains under threshold of %.2f\n"
+                          "and size of %d exceeds threshold of %d.",
+                          i, nonlinsys[i].equationIndex, nnz/(double)(size*size), nonlinearSparseSolverMaxDensity,
+                          size, nonlinearSparseSolverMinSize);
+        } else {
+          infoStreamPrint(LOG_STDOUT, 0,
+                          "Using sparse solver kinsol for nonlinear system %d (%ld),\n"
+                          "because density of %.2f remains under threshold of %.2f.",
+                          i, nonlinsys[i].equationIndex, nnz/(double)(size*size), nonlinearSparseSolverMaxDensity);
+        }
+      } else if (size > nonlinearSparseSolverMinSize) {
+        nonlinsys[i].nlsMethod = NLS_KINSOL;
+        nonlinsys[i].nlsLinearSolver = NLS_LS_KLU;
+        someBigSize = 1;
+        infoStreamPrint(LOG_STDOUT, 0,
+                        "Using sparse solver kinsol for nonlinear system %d (%ld),\n"
+                        "because size of %d exceeds threshold of %d.",
+                        i, nonlinsys[i].equationIndex, size, nonlinearSparseSolverMinSize);
+      }
+    }
+#endif
+
+    /* allocate stuff depending on the chosen method */
+    switch(nonlinsys[i].nlsMethod)
     {
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_HYBRID:
@@ -455,7 +490,7 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
         // solverData->ordinaryData = nonlinsys[i].solverData;
         allocateHomotopyData(size-1, &(solverData->initHomotopyData));
       } else {
-        nlsKinsolAllocate(size, &nonlinsys[i], data->simulationInfo->nlsLinearSolver);
+        nlsKinsolAllocate(size, &nonlinsys[i], nonlinsys[i].nlsLinearSolver);
         solverData->ordinaryData = nonlinsys[i].solverData;
       }
       nonlinsys[i].solverData = (void*) solverData;
@@ -470,15 +505,6 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
       }
       nonlinsys[i].solverData = (void*) solverData;
       break;
-#endif
-    case NLS_HOMOTOPY:
-      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
-        allocateHomotopyData(size-1, &nonlinsys[i].solverData);
-      } else {
-        allocateHomotopyData(size, &nonlinsys[i].solverData);
-      }
-      break;
-#if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_MIXED:
       mixedSolverData = (struct dataMixedSolver*) malloc(sizeof(struct dataMixedSolver));
       if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
@@ -491,10 +517,32 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
       nonlinsys[i].solverData = (void*) mixedSolverData;
       break;
 #endif
+    case NLS_HOMOTOPY:
+      if (nonlinsys[i].homotopySupport && (data->callback->useHomotopy == 2 || data->callback->useHomotopy == 3)) {
+        allocateHomotopyData(size-1, &nonlinsys[i].solverData);
+      } else {
+        allocateHomotopyData(size, &nonlinsys[i].solverData);
+      }
+      break;
     default:
       throwStreamPrint(threadData, "unrecognized nonlinear solver");
     }
   }
+
+  /* print relevant flag information */
+  if(someSmallDensity) {
+    if(someBigSize) {
+      infoStreamPrint(LOG_STDOUT, 0, "The maximum density and the minimal system size for using sparse solvers can be\n"
+                                     "specified using the runtime flags '<-nlssMaxDensity=value>' and '<-nlssMinSize=value>'.");
+    } else {
+      infoStreamPrint(LOG_STDOUT, 0, "The maximum density for using sparse solvers can be specified\n"
+                                     "using the runtime flag '<-nlssMaxDensity=value>'.");
+    }
+  } else if(someBigSize) {
+    infoStreamPrint(LOG_STDOUT, 0, "The minimal system size for using sparse solvers can be specified\n"
+                                   "using the runtime flag '<-nlssMinSize=value>'.");
+  }
+
   messageClose(LOG_NLS);
 
   TRACE_POP
@@ -563,7 +611,7 @@ int freeNonlinearSystems(DATA *data, threadData_t *threadData)
     }
 #endif
     /* free solver data */
-    switch(data->simulationInfo->nlsMethod)
+    switch(nonlinsys[i].nlsMethod)
     {
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_HYBRID:
@@ -813,7 +861,7 @@ int solveNLS(DATA *data, threadData_t *threadData, int sysNumber)
   struct dataMixedSolver *mixedSolverData;
 
   /* use the selected solver for solving nonlinear system */
-  switch(data->simulationInfo->nlsMethod)
+  switch(nonlinsys->nlsMethod)
   {
 #if !defined(OMC_MINIMAL_RUNTIME)
   case NLS_HYBRID:
@@ -930,7 +978,7 @@ int solveWithInitHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
   struct dataMixedSolver *mixedSolverData;
 
   /* use the homotopy solver for solving the initial system */
-  switch(data->simulationInfo->nlsMethod)
+  switch(nonlinsys->nlsMethod)
   {
 #if !defined(OMC_MINIMAL_RUNTIME)
   case NLS_HYBRID:
@@ -989,7 +1037,7 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
   double originalLambda = data->simulationInfo->lambda;
 
 #if !defined(OMC_MINIMAL_RUNTIME)
-  kinsol = (data->simulationInfo->nlsMethod == NLS_KINSOL);
+  kinsol = (nonlinsys->nlsMethod == NLS_KINSOL);
 #endif
 
   data->simulationInfo->currentNonlinearSystemIndex = sysNumber;
@@ -1058,7 +1106,6 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
       if (!homotopyDeactivated && !omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY])
         infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Try to solve nonlinear initial system %d without homotopy first.", sysNumber);
 
-      data->simulationInfo->lambda = 1.0;
       /* SOLVE! */
       nonlinsys->solved = solveNLS(data, threadData, sysNumber);
     }
