@@ -45,9 +45,12 @@
 #include "stylesheet.h"
 #include "inputcell.h"
 #include "notebookcommands.h"
+#include <QSplashScreen>
 
 #include <cstdlib>
 
+#include "../../../OMCompiler/Compiler/runtime/settingsimpl.h"
+
 namespace IAEX
 {
   /*!
@@ -83,14 +86,60 @@ namespace IAEX
 
     // ******************************************************
 
-  CellApplication::CellApplication( int &argc, char *argv[] )
+    class MyApp : public QApplication {
+    private:
+      CellApplication * ca = NULL;
+      public:
+        MyApp(int& argc, char**argv, CellApplication * c): QApplication(argc, argv)
+        {ca = c;}
+        bool event(QEvent *event) {
+          switch(event->type())
+          {
+            case QEvent::FileOpen:
+            {
+              QFileOpenEvent * fileOpenEvent = static_cast<QFileOpenEvent *>(event);
+              if(fileOpenEvent) {
+                ca->FileOpenEventTriggered = true;
+                ca->open(fileOpenEvent->file());
+
+                return true;
+              }
+            }
+            default:
+              return QApplication::event(event);
+          }
+        }
+    };
+
+  CellApplication::CellApplication(int &argc, char *argv[], threadData_t *threadData)
     : QObject()
   {
-    app_ = new QApplication(argc, argv);
+    app_ = new MyApp(argc, argv, this);
+
+
+    const char *installationDirectoryPath = SettingsImpl__getInstallationDirectoryPath();
+    if (!installationDirectoryPath) {
+      QMessageBox::critical(0, tr("Error"), tr("Could not find installation directory path. Please make sure OpenModelica is installed properly."), "OK");
+      app_->quit();
+      exit(1);
+    }
+
+    QString translationDirectory = installationDirectoryPath + QString("/share/omnotebook/nls");
+    // install Qt's default translations
+  #ifdef Q_OS_WIN
+    qtTranslator.load("qt_" + QLocale::system().name(), translationDirectory);
+  #else
+    qtTranslator.load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
+  #endif
+    app_->installTranslator(&qtTranslator);
+    // install application translations
+    translator.load("OMNotebook_" + QLocale::system().name(), translationDirectory);
+    app_->installTranslator(&translator);
+
     mainWindow = new QMainWindow();
     QDir dir;
 
-    // when last window closed, the applicaiton should quit also
+    // when last window closed, the application should quit also
     QObject::connect(app_, SIGNAL(lastWindowClosed()), app_, SLOT(quit()));
 
     //Create a commandCenter.
@@ -99,36 +148,30 @@ namespace IAEX
     /* Force C-style doubles */
     setlocale(LC_NUMERIC, "C");
 
-    // 2006-04-10 AF, use environment variable to find xml files
-    QString openmodelica = OmcInteractiveEnvironment::OpenModelicaHome();
-
-    if(!QDir().exists(openmodelica))
-    {
-      QMessageBox::critical( 0, "OpenModelica Error", "The environment variable OPENMODELICAHOME="+openmodelica+" is not a valid directory" );
-      exit(1);
-    }
-
+    /* Don't move this line
+     * Is important for threadData initialization
+     */
+    OmcInteractiveEnvironment *env = OmcInteractiveEnvironment::getInstance(threadData);
     // Avoid cluttering the whole disk with omc temp-files
-    OmcInteractiveEnvironment *env = OmcInteractiveEnvironment::getInstance();
     env->evalExpression("setCommandLineOptions(\"+d=shortOutput\")");
     QString cmdLine = env->getResult();
-    cout << "Set shortOutput flag: " << cmdLine.toStdString() << std::endl;
+    //cout << "Set shortOutput flag: " << cmdLine.toStdString() << std::endl;
     QString tmpDir = OmcInteractiveEnvironment::TmpPath();
     if (!QDir().exists(tmpDir)) QDir().mkdir(tmpDir);
     tmpDir = QDir(tmpDir).canonicalPath();
-    cout << "Temp.Dir " << tmpDir.toStdString() << std::endl;
+    //cout << "Temp.Dir " << tmpDir.toStdString() << std::endl;
     QString cdCmd = "cd(\"" + tmpDir + "\")";
     env->evalExpression(cdCmd);
     QString cdRes = env->getResult();
     cdRes.remove("\"");
     if (0 != tmpDir.compare(cdRes)) {
-      QMessageBox::critical( 0, "OpenModelica Error", "Could not create or cd to temp-dir\nCommand:\n  "+tmpDir+"\nReturned:\n  "+cdRes);
+      QMessageBox::critical( 0, "OpenModelica Error", tr("Could not create or cd to temp-dir\nCommand:\n  %1\nReturned:\n  %2").arg(tmpDir).arg(cdRes));
       exit(1);
     }
 
     // 2005-12-17 AF, Create instance (load styles) of stylesheet
-    // 2006-04-10 AF, use environment variable to find stylesheet.xml
-    Stylesheet *sheet;
+    // 2006-04-10 AF, use installation directory path to find stylesheet.xml
+    QString openmodelica = QString(installationDirectoryPath);
     try
     {
       QString stylesheetfile;
@@ -137,11 +180,11 @@ namespace IAEX
       else
         stylesheetfile = openmodelica + "/share/omnotebook/stylesheet.xml";
 
-      sheet = Stylesheet::instance( stylesheetfile );
+      Stylesheet::instance( stylesheetfile );
     }
     catch( exception &e )
     {
-      QMessageBox::warning( 0, "Error", e.what(), "OK" );
+      QMessageBox::warning( 0, tr("Error"), e.what(), "OK" );
       exit(-1);
     }
 
@@ -161,7 +204,7 @@ namespace IAEX
     {
       QString msg = e.what();
       msg += "\nCould not create command completion class, exiting OMNotebook";
-      QMessageBox::warning( 0, "Error", msg, "OK" );
+      QMessageBox::warning( 0, tr("Error"), msg, "OK" );
       std::exit(-1);
     }
 
@@ -185,37 +228,49 @@ namespace IAEX
         else
         {
           cout << "File not found: " << fileToOpen.toStdString() << endl;
-          open(QString::null);
+          open(QString());
         }
       }
       else
       {
-        // 2006-02-27 AF, use environment variable to find DrModelica
-        // 2006-03-24 AF, First try to find DrModelica.onb, then .nb
-        QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
-        //QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/QuickTour/HelloWorld.onb";
+        QIcon icon(":/Resources/OMNotebook_icon.svg");
+        QSplashScreen splash(icon.pixmap(300,400));
+        splash.show();
+        app_->processEvents();
+        splash.finish(mainWindow);
+        if (FileOpenEventTriggered)
+        {
 
-        if( dir.exists( drmodelica ))
-          open(drmodelica);
-        else if( dir.exists( "DrModelica/DrModelica.onb" ))
-          open( "DrModelica/DrModelica.onb" );
+        }
         else
         {
-          cout << "Unable to find (1): " << drmodelica.toStdString() << endl;
-          cout << "Unable to find (2): DrModelica/DrModelica.onb" << endl;
-
-          // NB
-          drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
+          // 2006-02-27 AF, use environment variable to find DrModelica
+          // 2006-03-24 AF, First try to find DrModelica.onb, then .nb
+          QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
+          //QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/QuickTour/HelloWorld.onb";
 
           if( dir.exists( drmodelica ))
             open(drmodelica);
-          else if( dir.exists( "DrModelica/DrModelica.nb" ))
-            open( "DrModelica/DrModelica.nb" );
+          else if( dir.exists( "DrModelica/DrModelica.onb" ))
+            open( "DrModelica/DrModelica.onb" );
           else
           {
-            cout << "Unable to find (3): " << drmodelica.toStdString() << endl;
-            cout << "Unable to find (4): DrModelica/DrModelica.nb" << endl;
-            open(QString::null);
+            cout << "Unable to find (1): " << drmodelica.toStdString() << endl;
+            cout << "Unable to find (2): DrModelica/DrModelica.onb" << endl;
+
+            // NB
+            drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
+
+            if( dir.exists( drmodelica ))
+              open(drmodelica);
+            else if( dir.exists( "DrModelica/DrModelica.nb" ))
+              open( "DrModelica/DrModelica.nb" );
+            else
+            {
+              cout << "Unable to find (3): " << drmodelica.toStdString() << endl;
+              cout << "Unable to find (4): DrModelica/DrModelica.nb" << endl;
+              open(QString());
+            }
           }
         }
       }
@@ -248,8 +303,7 @@ namespace IAEX
     {
       if( !dir.remove( removeList_.at(i) ))
       {
-        QString msg = "Could not remove temporary image " + removeList_.at(i) + " from harddrive.";
-        QMessageBox::warning( 0, "Warning", msg, "OK" );
+        QMessageBox::warning( 0, tr("Warning"), tr("Could not remove temporary image %1 from harddrive.").arg(removeList_.at(i)), "OK" );
       }
     }
   }
@@ -336,7 +390,7 @@ namespace IAEX
    * \author Ingemar Axelsson and Anders Fernström
    * \date 2006-05-03 (update)
    *
-   * \brief Open an file, and display the content of hte file
+   * \brief Open an file, and display the content of the file
    *
    * 2005-09-22 AF, added the filename to the NotebookWindow() call
    * 2005-10-11 AF, Porting, added resize call, so all cells get the
@@ -364,7 +418,7 @@ namespace IAEX
       DocumentView *v = new NotebookWindow(d, filename);
       add(v);
 
-      // 2006-01-31 AF, Open window minimized insted of normal
+      // 2006-01-31 AF, Open window minimized instead of normal
 
       //v->showMinimized();
 
@@ -379,8 +433,20 @@ namespace IAEX
 
       // 2006-01-31 AF, show window again
       v->show();
+      v->raise();  // for MacOS
+      v->activateWindow(); // for Windows
 
-      v->resize( 801, 600 ); //fjass
+      vector<DocumentView *> windowViews = documentViewList();
+      vector<DocumentView *>::iterator v_iter = windowViews.begin();
+      while( v_iter != windowViews.end() )
+      {
+        ((NotebookWindow *)*v_iter)->updateWindowMenu();
+        ++v_iter;
+      }
+
+      QDesktopWidget dw;
+      v->move(0, 0);
+      v->resize(dw.geometry().width(),dw.geometry().height());
 
       // 2005-11-30 AF, apply hide() and show() to closed groupcells
       // childs in the documentview
@@ -398,7 +464,7 @@ namespace IAEX
   * \date 2006-01-16
   *
   * \brief Add filename to a list of temporary files that should
-  * be deleted when the applicaiton quits.
+  * be deleted when the application quits.
   */
   void CellApplication::removeTempFiles(QString filename)
   {
@@ -409,7 +475,7 @@ namespace IAEX
   * \author Anders Fernström
   * \date 2006-01-27
   *
-  * \brief returns list of all current doucment views
+  * \brief returns list of all current document views
   */
   vector<DocumentView *> CellApplication::documentViewList()
   {
@@ -448,6 +514,12 @@ namespace IAEX
       else
         ++dv_iter;
     }
+    dv_iter = views_.begin();
+    while( dv_iter != views_.end() )
+    {
+      ((NotebookWindow *)*dv_iter)->updateWindowMenu();
+      ++dv_iter;
+    }
   }
 
   /*!
